package com.sbh.background_locations.adapters

import android.content.Context
import android.util.Log
import com.sbh.background_locations.utils.SharedPreferenceUtil
import com.sbh.background_locations.utils.SingletonHolder
import org.json.JSONException
import org.json.JSONObject

class SBHConfig() {
    private lateinit var mContext: Context
    private var mIsEnabled: Boolean = false
    private var mSchedulerEnabled: Boolean = false

    companion object : SingletonHolder<SBHConfig, Context>(::SBHConfig) {
        const val TAG = "SBHConfig"
    }

    constructor(context: Context) : this() {
        mContext = context
    }

    fun updateWithJSONObject(json: JSONObject) {
        Log.d(TAG, "updateWithJSONObject => $json")
        SharedPreferenceUtil.setConfig(mContext, json)
    }

    //
//    fun isFirstBoot(): Boolean {
//        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
//    }
//
//    fun reset() {
//        TODO("not implemented") //To change body of created functions use File | Settings | File Templates.
//    }
//
    fun toJson(): Any {
        val con = SharedPreferenceUtil.getConfig(mContext)
        if (con != null) {
            return try {
                Log.d(TAG, "toJson => $con")
                JSONObject(con)
            } catch (e: JSONException) {
                Log.e(TAG, "toJson => ${e.message}", e)
                JSONObject.NULL
            }
        } else {
            Log.d(TAG, "toJson => con is null")
        }
        return JSONObject.NULL
    }

    fun getEnabled(): Boolean {
        return mIsEnabled
    }

    fun hasSchedule(): Boolean {
        return false
    }

    fun getSchedulerEnabled(): Boolean {
        return mSchedulerEnabled
    }

    fun setSchedulerEnabled(b: Boolean) {
        mSchedulerEnabled = b
    }

    fun isLocationTrackingMode(): Boolean {
        return false
    }

    fun firebaseConfig(): SBHFirebaseConfig? {
        val json = toJson()
        return if (json is JSONObject && json.has("firebase_config") && !json.isNull("firebase_config")) {
            SBHFirebaseConfig.fromJson(json.getJSONObject("firebase_config"))
        } else {
            Log.d(TAG, "firebaseConfig => json is null")
            null
        }
    }

    fun reset() {
        SharedPreferenceUtil.resetConfig(mContext)
    }
}