package com.sbh.background_locations.common

class Coordinates {
    companion object {
        private const val EARTH_RADIUS = 6371.0 * 1000.0 // meters
        
        fun distanceBetween(lon1: Double, lat1: Double, lon2: Double, lat2: Double): Double {
            val deltaLon = Math.toRadians(lon2 - lon1)
            val deltaLat = Math.toRadians(lat2 - lat1)
            val a = Math.pow(Math.sin(deltaLat / 2.0), 2.0) + Math.cos(Math.toRadians(lat1)) *
                    Math.cos(Math.toRadians(lat2)) *
                    Math.pow(Math.sin(deltaLon / 2.0), 2.0)
            val c = 2.0 * Math.atan2(Math.sqrt(a), Math.sqrt(1.0 - a))
            return EARTH_RADIUS * c
        }

        fun longitudeToMeters(lon: Double): Double {
            val distance = distanceBetween(lon, 0.0, 0.0, 0.0)
            return distance * if (lon < 0.0) -1.0 else 1.0
        }

        fun metersToGeoPoint(lonMeters: Double,
                             latMeters: Double): GeoPoint {
            val point = GeoPoint(0.0, 0.0)
            val pointEast = pointPlusDistanceEast(point, lonMeters)
            return pointPlusDistanceNorth(pointEast, latMeters)
        }

        fun latitudeToMeters(lat: Double): Double {
            val distance = distanceBetween(0.0, lat, 0.0, 0.0)
            return distance * if (lat < 0.0) -1.0 else 1.0
        }

        private fun getPointAhead(point: GeoPoint,
                                  distance: Double,
                                  azimuthDegrees: Double): GeoPoint {

            val radiusFraction = distance / EARTH_RADIUS
            val bearing = Math.toRadians(azimuthDegrees)
            val lat1 = Math.toRadians(point.latitude)
            val lng1 = Math.toRadians(point.longitude)

            val lat2_part1 = Math.sin(lat1) * Math.cos(radiusFraction)
            val lat2_part2 = Math.cos(lat1) * Math.sin(radiusFraction) * Math.cos(bearing)
            val lat2 = Math.asin(lat2_part1 + lat2_part2)

            val lng2_part1 = Math.sin(bearing) * Math.sin(radiusFraction) * Math.cos(lat1)
            val lng2_part2 = Math.cos(radiusFraction) - Math.sin(lat1) * Math.sin(lat2)
            var lng2 = lng1 + Math.atan2(lng2_part1, lng2_part2)

            lng2 = (lng2 + 3.0 * Math.PI) % (2.0 * Math.PI) - Math.PI

            return GeoPoint(Math.toDegrees(lat2), Math.toDegrees(lng2))
        }

        private fun pointPlusDistanceEast(point: GeoPoint, distance: Double): GeoPoint {
            return getPointAhead(point, distance, 90.0)
        }

        private fun pointPlusDistanceNorth(point: GeoPoint, distance: Double): GeoPoint {
            return getPointAhead(point, distance, 0.0)
        }

        fun calculateDistance(track: Array<GeoPoint>?): Double {
            var distance = 0.0
            var lastLon: Double
            var lastLat: Double
            //WARNING! I didn't find array.length type. Seems it's int, so we can use next comparison:
            if (track == null || track.size - 1 <= 0)
            //track.length == 0 || track.length == 1
                return 0.0

            lastLon = track[0].longitude
            lastLat = track[0].latitude

            for (i in 1 until track.size) {
                distance += Coordinates.distanceBetween(
                        lastLat, lastLon,
                        track[i].latitude, track[i].longitude)
                lastLat = track[i].latitude
                lastLon = track[i].longitude
            }
            return distance
        }
    }
}
