package com.sbh.background_locations.common

import androidx.annotation.NonNull

class SensorGpsDataItem() : Comparable<SensorGpsDataItem> {
    internal var timestamp: Long = 0
    internal var gpsLat: Double = 0.toDouble()
    internal var gpsLon: Double = 0.toDouble()
    internal var gpsAlt: Double = 0.toDouble()
    internal var absNorthAcc: Float = 0f
    internal var absEastAcc: Float = 0f
    internal var absUpAcc: Float = 0f
    internal var speed: Float = 0f
    internal var course: Float = 0f
    internal var posErr: Float = 0f
    internal var velErr: Double = 0.toDouble()

    companion object {
        val NOT_INITIALIZED = 361.0f
    }

    constructor(timestamp: Long,
                gpsLat: Double, gpsLon: Double, gpsAlt: Double,
                absNorthAcc: Float, absEastAcc: Float, absUpAcc: Float,
                speed: Float, course: Float,
                posErr: Float, velErr: Double,
                declination: Double) : this() {
        this.timestamp = timestamp
        this.gpsLat = gpsLat
        this.gpsLon = gpsLon
        this.gpsAlt = gpsAlt
        this.absNorthAcc = absNorthAcc
        this.absEastAcc = absEastAcc
        this.absUpAcc = absUpAcc
        this.speed = speed
        this.course = course
        this.posErr = posErr
        this.velErr = velErr

        this.absNorthAcc = (absNorthAcc * Math.cos(declination) + absEastAcc * Math.sin(declination)).toFloat()
        this.absEastAcc = (absEastAcc * Math.cos(declination) - absNorthAcc * Math.sin(declination)).toFloat()
    }

    fun getTimestamp(): Long {
        return timestamp
    }

    fun getGpsLat(): Double {
        return gpsLat
    }

    fun getGpsLon(): Double {
        return gpsLon
    }

    fun getGpsAlt(): Double {
        return gpsAlt
    }

    fun getAbsNorthAcc(): Float {
        return absNorthAcc
    }

    fun getAbsEastAcc(): Float {
        return absEastAcc
    }

    fun getAbsUpAcc(): Float {
        return absUpAcc
    }

    fun getSpeed(): Float {
        return speed
    }

    fun getCourse(): Float {
        return course
    }

    fun getPosErr(): Float {
        return posErr
    }

    fun getVelErr(): Double {
        return velErr
    }

    override fun compareTo(@NonNull o: SensorGpsDataItem): Int {
        return (this.timestamp - o.timestamp).toInt()
    }
}
