package com.sbh.background_locations.filters

import android.location.Location
import com.sbh.background_locations.common.Coordinates
import com.sbh.background_locations.common.GeoPoint

class GeohashRTFilter() {
    var PROVIDER_NAME = "GeoHashFiltered"

    private var m_distanceGeoFiltered = 0.0
    private var m_distanceGeoFilteredHP = 0.0
    private var m_distanceAsIs = 0.0
    private var m_distanceAsIsHP = 0.0

    private val COORD_NOT_INITIALIZED = 361.0
    private var ppCompGeoHash = 0
    private var ppReadGeoHash = 1

    private var geoHashBuffers: LongArray? = null
    private var pointsInCurrentGeohashCount: Int = 0

    private var currentGeoPoint: GeoPoint? = null
    private var lastApprovedGeoPoint: GeoPoint? = null
    private var lastGeoPointAsIs: GeoPoint? = null

    private var m_geoFilteredTrack: MutableList<Location> = ArrayList()
    fun getGeoFilteredTrack(): List<Location> {
        return m_geoFilteredTrack
    }


    private var isFirstCoordinate: Boolean = true
    private var m_geohashPrecision: Int = 0
    private var m_geohashMinPointCount: Int = 0

    constructor(geohashPrecision: Int,
                geohashMinPointCount: Int) : this() {
        m_geohashPrecision = geohashPrecision
        m_geohashMinPointCount = geohashMinPointCount
        m_geoFilteredTrack = ArrayList()
        reset()
    }

    fun getDistanceGeoFiltered(): Double {
        return m_distanceGeoFiltered
    }

    fun getDistanceGeoFilteredHP(): Double {
        return m_distanceGeoFilteredHP
    }

    fun getDistanceAsIs(): Double {
        return m_distanceAsIs
    }

    fun getDistanceAsIsHP(): Double {
        return m_distanceAsIsHP
    }

//    private var m_logger: ILogger? = null

    fun reset() {
//        m_logger = logger
        m_geoFilteredTrack.clear()
        geoHashBuffers = LongArray(2)
        pointsInCurrentGeohashCount = 0
        lastApprovedGeoPoint = GeoPoint(COORD_NOT_INITIALIZED, COORD_NOT_INITIALIZED)
        currentGeoPoint = GeoPoint(COORD_NOT_INITIALIZED, COORD_NOT_INITIALIZED)

        lastGeoPointAsIs = GeoPoint(COORD_NOT_INITIALIZED, COORD_NOT_INITIALIZED)
        m_distanceGeoFiltered = 0.0
        m_distanceGeoFilteredHP = m_distanceGeoFiltered
        m_distanceAsIs = 0.0
        m_distanceAsIsHP = m_distanceAsIs
        isFirstCoordinate = true
    }

    private val hpResBuffAsIs = FloatArray(3)
    private val hpResBuffGeo = FloatArray(3)

    fun filter(loc: Location) {
//        if (m_logger != null) {
//            val toLog = String.format("%d%d FKS : lat=%f, lon=%f, alt=%f",
//                    Utils.LogMessageType.FILTERED_GPS_DATA.ordinal,
//                    loc.time,
//                    loc.latitude, loc.longitude, loc.altitude)
//            m_logger!!.log2file(toLog)
//        }

        val pi = GeoPoint(loc.latitude, loc.longitude)
        if (isFirstCoordinate) {
            geoHashBuffers?.set(ppCompGeoHash, GeoHash.encode_u64(pi.latitude, pi.longitude, m_geohashPrecision))
            currentGeoPoint!!.latitude = pi.latitude
            currentGeoPoint!!.longitude = pi.longitude
            pointsInCurrentGeohashCount = 1

            isFirstCoordinate = false
            lastGeoPointAsIs!!.latitude = pi.latitude
            lastGeoPointAsIs!!.longitude = pi.longitude
            return
        }

        m_distanceAsIs += Coordinates.distanceBetween(
                lastGeoPointAsIs!!.longitude,
                lastGeoPointAsIs!!.latitude,
                pi.longitude,
                pi.latitude)

        Location.distanceBetween(
                lastGeoPointAsIs!!.latitude,
                lastGeoPointAsIs!!.longitude,
                pi.latitude,
                pi.longitude,
                hpResBuffAsIs)


        m_distanceAsIsHP += hpResBuffAsIs[0].toDouble()
        lastGeoPointAsIs!!.longitude = loc.longitude
        lastGeoPointAsIs!!.latitude = loc.latitude

        geoHashBuffers?.set(ppReadGeoHash, GeoHash.encode_u64(pi.latitude, pi.longitude, m_geohashPrecision))
        if (geoHashBuffers!![ppCompGeoHash] != geoHashBuffers!![ppReadGeoHash]) {
            if (pointsInCurrentGeohashCount >= m_geohashMinPointCount) {
                currentGeoPoint!!.latitude /= pointsInCurrentGeohashCount
                currentGeoPoint!!.longitude /= pointsInCurrentGeohashCount

                if (lastApprovedGeoPoint!!.latitude !== COORD_NOT_INITIALIZED) {
                    val dd1 = Coordinates.distanceBetween(
                            lastApprovedGeoPoint!!.longitude,
                            lastApprovedGeoPoint!!.latitude,
                            currentGeoPoint!!.longitude,
                            currentGeoPoint!!.latitude)
                    m_distanceGeoFiltered += dd1
                    Location.distanceBetween(
                            lastApprovedGeoPoint!!.latitude,
                            lastApprovedGeoPoint!!.longitude,
                            currentGeoPoint!!.latitude,
                            currentGeoPoint!!.longitude,
                            hpResBuffGeo)
                    val dd2 = hpResBuffGeo[0].toDouble()
                    m_distanceGeoFilteredHP += dd2
                }
                lastApprovedGeoPoint!!.longitude = currentGeoPoint!!.longitude
                lastApprovedGeoPoint!!.latitude = currentGeoPoint!!.latitude
                val laLoc = Location(PROVIDER_NAME)
                laLoc.latitude = lastApprovedGeoPoint!!.latitude
                laLoc.longitude = lastApprovedGeoPoint!!.longitude
                laLoc.altitude = loc.altitude //hack.
                laLoc.time = loc.time //hack2
                m_geoFilteredTrack.add(laLoc)
                currentGeoPoint!!.longitude = 0.0
                currentGeoPoint!!.latitude = currentGeoPoint!!.longitude
            }

            pointsInCurrentGeohashCount = 1
            currentGeoPoint!!.latitude = pi.latitude
            currentGeoPoint!!.longitude = pi.longitude
            //swap buffers
            val swp = ppCompGeoHash
            ppCompGeoHash = ppReadGeoHash
            ppReadGeoHash = swp
            return
        }

        currentGeoPoint!!.latitude += pi.latitude
        currentGeoPoint!!.longitude += pi.longitude
        ++pointsInCurrentGeohashCount
    }

    fun stop() {
        if (pointsInCurrentGeohashCount >= m_geohashMinPointCount) {
            currentGeoPoint!!.latitude /= pointsInCurrentGeohashCount
            currentGeoPoint!!.longitude /= pointsInCurrentGeohashCount

            if (lastApprovedGeoPoint!!.latitude !== COORD_NOT_INITIALIZED) {
                val dd1 = Coordinates.distanceBetween(
                        lastApprovedGeoPoint!!.longitude,
                        lastApprovedGeoPoint!!.latitude,
                        currentGeoPoint!!.longitude,
                        currentGeoPoint!!.latitude)
                m_distanceGeoFiltered += dd1
                Location.distanceBetween(
                        lastApprovedGeoPoint!!.latitude,
                        lastApprovedGeoPoint!!.longitude,
                        currentGeoPoint!!.latitude,
                        currentGeoPoint!!.longitude,
                        hpResBuffGeo)
                val dd2 = hpResBuffGeo[0].toDouble()
                m_distanceGeoFilteredHP += dd2
            }
            lastApprovedGeoPoint!!.longitude = currentGeoPoint!!.longitude
            lastApprovedGeoPoint!!.latitude = currentGeoPoint!!.latitude
            val laLoc = Location(PROVIDER_NAME)
            laLoc.latitude = lastApprovedGeoPoint!!.latitude
            laLoc.longitude = lastApprovedGeoPoint!!.longitude
            m_geoFilteredTrack.add(laLoc)
            currentGeoPoint!!.longitude = 0.0
            currentGeoPoint!!.latitude = currentGeoPoint!!.longitude
        }
    }
}